/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : UART0.c
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : This file implements device driver for Serial module.
* Creation Date: 2016/08/07
*******************************************************************************/

/*******************************************************************************
Includes
*******************************************************************************/
#include "r_cg_macrodriver.h"
#include "UART0.h"
#include "r_cg_userdefine.h"

/*******************************************************************************
Pragma directive
*******************************************************************************/

/*******************************************************************************
Global variables and functions
*******************************************************************************/

/*******************************************************************************
* Function Name: R_UART0_Start
* Description  : UART0̓
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_UART0_Start(void)
{
    SO0 |= 0x01;                                /* o̓x̓nCx   */
    SOE0 |= 0x01;                               /* VAo͋         */
    SS0 |= 0b00000011;                          /* UART0 M싖     */
    STIF0 = 1U;                                 /* INTST0荞ݗvZbg */
    SRIF0 = 0U;                                 /* INTSR0荞ݗvNA */
}


#ifndef INTFLAG_MODE
/*******************************************************************************

    staus(BFF bit) polling mode

*******************************************************************************/

/*******************************************************************************
* Function Name: uart0_poll_Rx
* Description  : UART0̎MXe[^XmFCf[^Γǂݏo
* Arguments    : None
* Return Value : ʐMXe[^XƎMf[^i΁j
*               bit15 : Mf[^Ȃi1j^Mf[^i0j
*               bit10 : t~OEG[
*               bit9  : peBEG[
*               bit8  : I[o[EG[
*               bit7-0: Mf[^
*******************************************************************************/
uint16_t uart0_poll_Rx(void)
{
    uint16_t work;                              /* Ɨpϐ               */

    work = SSR01;                               /* MXe[^Xǂݏo   */

    if ( work & 0x20 )              /* M                           */
    {
        SIR01 = ( work & 0x07 );                /* G[EtONA   */
        work = ( work & 0x07 ) << 8;            /* Xe[^XʂɃVtg */
        work |= RXD0;                           /* Mf[^ǂݏo     */
    }
    else
    {                               /* MĂȂƂ             */
        work = 0x8000;                          /* not readyZbg        */
    }

    return(work);
}


/*******************************************************************************
* Function Name: uart0_Rx
* Description  : UART0̎M҂CMf[^ǂݏoB
* Arguments    : None
* Return Value : ʐMXe[^XƎMf[^
*               bit10 : t~OEG[
*               bit9  : peBEG[
*               bit8  : I[o[EG[
*               bit7-0: Mf[^
*******************************************************************************/
uint16_t uart0_Rx(void)
{
    uint16_t work;                              /* Ɨpϐ               */

    work = SSR01;                               /* MXe[^Xǂݏo   */

    while ( 0 == ( work & 0x20 ) )  /* M҂                         */
    {
        work = SSR01;
    }
        
    work &= 0x07;                               /* G[EtOȊO}XN */
    SIR01 = work;                               /* G[EtONA   */
    work = work << 8;                           /* 8rbgɈړ        */
    work |= RXD0;                               /* Mf[^̓ǂݏo     */

    return(work);                               /* Mf[^߂l     */

}

/*******************************************************************************
* Function Name: uart0_poll_Tx
* Description  : UART0̑MXe[^XmFCM\Ȃ瑗M
* Arguments    : Mf[^
* Return Value : ʐMXe[^XƑMf[^
*               bit15 : MJni1j^MJni0j
*               bit7-0: Mf[^^MJn0x00
*******************************************************************************/
uint16_t uart0_poll_Tx( uint8_t tx_data )
{

    uint16_t work;                              /* Ɨpϐ               */

    work = ( SSR00 & 0x20 );                    /* Mobt@mF         */
    if ( 0 == work )
    {                               /* f[^M\                     */
        TXD0 = tx_data;                         /* f[^M               */
        work = 0x0000;                          /* ߂lNA           */
    }
    else
    {                               /* f[^Ms\                   */
        work = ( 0x8000 | tx_data );            /* Mf[^߂l     */
    }

    return(work);                               /* ߂lݒ             */

}

/*******************************************************************************
* Function Name: uart0_Tx
* Description  : UART0M\ɂȂ̂҂CM
* Arguments    : Mf[^
* Return Value : None
*******************************************************************************/
void uart0_Tx( uint8_t tx_data )
{

    while ( 0 != ( SSR00 & 0x20 ) ) /* M\ɂȂ̂҂               */
    {
        NOP();
    }

    TXD0 = tx_data;                             /* f[^MJn           */

}

#else
/*******************************************************************************

    interrupt request flag polling mode

*******************************************************************************/

/*******************************************************************************
* Function Name: uart0_poll_Rx
* Description  : UART0̎MXe[^XmFCf[^Γǂݏo
* Arguments    : None
* Return Value : ʐMXe[^XƎMf[^i΁j
*               bit15 : Mf[^Ȃi1j^Mf[^i0j
*               bit10 : t~OEG[
*               bit9  : peBEG[
*               bit8  : I[o[EG[
*               bit7-0: Mf[^
*******************************************************************************/
uint16_t uart0_poll_Rx(void)
{
    uint16_t work;                              /* Ɨpϐ               */


    if ( 1 == SRIF0 )               /* M                           */
    {
        SRIF0 = 0;                              /* 荞ݗvtONA */
        work = SSR01 & 0x07;                    /* MXe[^Xǂݏo   */
        SIR01 = work;                           /* G[EtONA   */
        work = work << 8;                       /* Xe[^XʂɃVtg */
        work |= RXD0;                           /* Mf[^ǂݏo     */
    }
    else
    {                               /* MĂȂƂ             */
        work = 0x8000;                          /* not readyZbg        */
    }

    return(work);
}


/*******************************************************************************
* Function Name: uart0_Rx
* Description  : UART0̎M҂CMf[^ǂݏoB
* Arguments    : None
* Return Value : ʐMXe[^XƎMf[^
*               bit10 : t~OEG[
*               bit9  : peBEG[
*               bit8  : I[o[EG[
*               bit7-0: Mf[^
*******************************************************************************/
uint16_t uart0_Rx(void)
{
    uint16_t work;                              /* Ɨpϐ               */


    while ( 0 == SRIF0 )            /* M҂                         */
    {
        NOP();
    }
    SRIF0 = 0;                                  /* 荞ݗvtONA */
    work = SSR01 & 0x07;                        /* MXe[^Xǂݏo   */
    SIR01 = work;                               /* G[EtONA   */
    work = work << 8;                           /* 8rbgɈړ        */
    work |= RXD0;                               /* Mf[^̓ǂݏo     */


    return(work);                               /* Mf[^߂l     */

}

/*******************************************************************************
* Function Name: uart0_poll_Tx
* Description  : UART0̑MXe[^XmFCM\Ȃ瑗M
* Arguments    : Mf[^
* Return Value : ʐMXe[^XƑMf[^
*               bit15 : MJni1j^MJni0j
*               bit7-0: Mf[^^MJn0x00
*******************************************************************************/
uint16_t uart0_poll_Tx( uint8_t tx_data )
{

    uint16_t work;                              /* Ɨpϐ               */

    if ( 1 == STIF0 )
    {                               /* f[^M\                     */
        TXD0 = tx_data;                         /* f[^M               */
        work = 0x0000;                          /* ߂lNA           */
        STIF0 = 0;                              /* 荞ݗvtONA */
    }
    else
    {                               /* f[^Ms\                   */
        work = ( 0x8000 | tx_data );            /* Mf[^߂l     */
    }

    return(work);                               /* ߂lݒ             */

}

/*******************************************************************************
* Function Name: uart0_Tx
* Description  : UART0M\ɂȂ̂҂CM
* Arguments    : Mf[^
* Return Value : None
*******************************************************************************/
void uart0_Tx( uint8_t tx_data )
{

    while ( 0 == STIF0 )            /* M\ɂȂ̂҂               */
    {
        NOP();
    }

    TXD0 = tx_data;                             /* f[^MJn           */
    STIF0 = 0;                                  /* 荞ݗvtONA */

}

#endif
